{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 25,
   "id": "690927be",
   "metadata": {},
   "outputs": [],
   "source": [
    "#아래는 주피터 노트북에 맞게 단계별로 표시하고 소스 시각화까지 표현한 코드입니다.\n",
    "#1.라이브러리와 데이터셋 경로 설정\n",
    "#==== 1. Import libraries and set dataset path\n",
    "import os\n",
    "import json\n",
    "import numpy as np\n",
    "import torch\n",
    "import torch.nn as nn\n",
    "import torch.optim as optim\n",
    "from torch.utils.data import Dataset, DataLoader\n",
    "from tqdm.auto import tqdm\n",
    "import cv2\n",
    "import matplotlib.pyplot as plt\n",
    "from IPython.display import clear_output\n",
    "\n",
    "dataset_path = \"d:\\\\user\\\\dataset.json\"\n",
    "model_path = \"d:\\\\user\\\\model.pth\""
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 26,
   "id": "5ce0e209",
   "metadata": {},
   "outputs": [],
   "source": [
    "#2.데이터셋 불러오기\n",
    "#==== 2. Load dataset\n",
    "class CustomDataset(Dataset):\n",
    "    def __init__(self, dataset):\n",
    "        self.dataset = dataset\n",
    "\n",
    "    def __len__(self):\n",
    "        return len(self.dataset)\n",
    "\n",
    "    def __getitem__(self, idx):\n",
    "        image_path = self.dataset[idx][\"image_path\"]\n",
    "        mask_path = self.dataset[idx][\"mask_path\"]\n",
    "\n",
    "        image = cv2.imread(image_path, cv2.IMREAD_GRAYSCALE)\n",
    "        mask = cv2.imread(mask_path, cv2.IMREAD_GRAYSCALE)\n",
    "\n",
    "        _, mask = cv2.threshold(mask, 128, 1, cv2.THRESH_BINARY)\n",
    "\n",
    "        image = torch.tensor(image, dtype=torch.float32).unsqueeze(0)\n",
    "        mask = torch.tensor(mask, dtype=torch.long)\n",
    "\n",
    "        return image, mask\n",
    "\n",
    "with open(dataset_path, \"r\") as f:\n",
    "    dataset = [json.loads(line) for line in f]\n",
    "\n",
    "num_samples = len(dataset)\n",
    "train_size = int(num_samples * 0.8)\n",
    "val_size = int(num_samples * 0.1)\n",
    "test_size = num_samples - train_size - val_size\n",
    "\n",
    "train_data, val_data, test_data = torch.utils.data.random_split(dataset, [train_size, val_size, test_size])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 27,
   "id": "973821c3",
   "metadata": {},
   "outputs": [],
   "source": [
    "#3.U-Net 모델 구현\n",
    "#==== 3. Implement U-Net model\n",
    "class UNet(nn.Module):\n",
    "    def __init__(self, in_channels, out_channels):\n",
    "        super(UNet, self).__init__()\n",
    "\n",
    "        # Contracting Path (Encoder)\n",
    "        self.encoder1 = nn.Sequential(\n",
    "            nn.Conv2d(in_channels, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(64, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "        self.pool1 = nn.MaxPool2d(2)\n",
    "\n",
    "        self.encoder2 = nn.Sequential(\n",
    "            nn.Conv2d(64, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(128, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "        self.pool2 = nn.MaxPool2d(2)\n",
    "\n",
    "        self.encoder3 = nn.Sequential(\n",
    "            nn.Conv2d(128, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(256, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "        self.pool3 = nn.MaxPool2d(2)\n",
    "\n",
    "        # Bottleneck\n",
    "        self.bottleneck = nn.Sequential(\n",
    "            nn.Conv2d(256, 512, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(512, 512, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "        \n",
    "        # Expanding Path (Decoder)\n",
    "        self.upconv3 = nn.ConvTranspose2d(512, 256, 2, stride=2)\n",
    "        self.decoder3 = nn.Sequential(\n",
    "            nn.Conv2d(512, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(256, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "\n",
    "        self.upconv2 = nn.ConvTranspose2d(256, 128, 2, stride=2)\n",
    "        self.decoder2 = nn.Sequential(\n",
    "            nn.Conv2d(256, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(128, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "\n",
    "        self.upconv1 = nn.ConvTranspose2d(128, 64, 2, stride=2)\n",
    "        self.decoder1 = nn.Sequential(\n",
    "            nn.Conv2d(128, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(64, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "\n",
    "        self.out_conv = nn.Conv2d(64, out_channels, 1)\n",
    "\n",
    "    def forward(self, x):\n",
    "        enc1 = self.encoder1(x)\n",
    "        enc2 = self.encoder2(self.pool1(enc1))\n",
    "        enc3 = self.encoder3(self.pool2(enc2))\n",
    "\n",
    "        bottleneck = self.bottleneck(self.pool3(enc3))\n",
    "\n",
    "        dec3 = self.decoder3(torch.cat((enc3, self.upconv3(bottleneck)), 1))\n",
    "        dec2 = self.decoder2(torch.cat((enc2, self.upconv2(dec3)), 1))\n",
    "        dec1 = self.decoder1(torch.cat((enc1, self.upconv1(dec2)), 1))\n",
    "\n",
    "        return self.out_conv(dec1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "id": "f8f326aa",
   "metadata": {},
   "outputs": [],
   "source": [
    "#여기까지의 코드는 U-Net 모델의 나머지 부분입니다.\n",
    "#병목 구간 다음에 확장 경로(디코더)를 구현한 것이며,\n",
    "#각 단계에서 두 개의 컨볼루션 계층과 활성화 함수 ReLU를 사용합니다.\n",
    "#이어서 전치 컨볼루션 계층을 사용하여 특성 맵의 크기를 늘리고(concatenate) 인코더와 디코더의 특성 맵을 연결합니다.\n",
    "#마지막으로 출력 컨볼루션 계층을 통해 예측 마스크를 생성합니다."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "4560455c",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": 30,
   "id": "d9a944b8",
   "metadata": {},
   "outputs": [],
   "source": [
    "#위 코드는 다음과 같은 과정을 수행합니다:\n",
    "#1.하이퍼파라미터 설정\n",
    "#2.데이터셋 및 데이터 로더 준비\n",
    "#3.U-Net 모델, 손실 함수, 옵티마이저 초기화\n",
    "#4.모델 학습 (주어진 에폭 동안 반복)\n",
    "#학습이 완료된 후, 테스트 데이터를 사용하여 성능을 평가하거나\n",
    "#새로운 이미지에 대한 세그멘테이션 마스크를 생성할 수 있습니다.\n",
    "#이 과정에서는 모델을 평가 모드로 전환하고, 데이터를 입력하여 예측 결과를 얻습니다.\n",
    "#이후, 원하는 성능 지표로 결과를 평가하거나 시각화할 수 있습니다."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "594fd26e",
   "metadata": {},
   "outputs": [],
   "source": [
    "import torch\n",
    "import torch.nn as nn\n",
    "\n",
    "class UNet(nn.Module):\n",
    "    def __init__(self, in_channels, out_channels):\n",
    "        super(UNet, self).__init__()\n",
    "\n",
    "        self.encoder = nn.Sequential(\n",
    "            nn.Conv2d(in_channels, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(64, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.MaxPool2d(2, 2)\n",
    "        )\n",
    "\n",
    "        self.middle = nn.Sequential(\n",
    "            nn.Conv2d(64, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(128, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.MaxPool2d(2, 2)\n",
    "        )\n",
    "\n",
    "        self.bottleneck = nn.Sequential(\n",
    "            nn.Conv2d(128, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(256, 256, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.MaxPool2d(2, 2)\n",
    "        )\n",
    "\n",
    "        self.upconv1 = nn.ConvTranspose2d(256, 128, 2, stride=2)\n",
    "        self.decoder1 = nn.Sequential(\n",
    "            nn.Conv2d(256, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(128, 128, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "\n",
    "        self.upconv2 = nn.ConvTranspose2d(128, 64, 2, stride=2)\n",
    "        self.decoder2 = nn.Sequential(\n",
    "            nn.Conv2d(128, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True),\n",
    "            nn.Conv2d(64, 64, 3, padding=1),\n",
    "            nn.ReLU(inplace=True)\n",
    "        )\n",
    "\n",
    "        self.output_conv = nn.Conv2d(64, out_channels, 1)\n",
    "\n",
    "    def forward(self, x):\n",
    "        enc1 = self.encoder(x)\n",
    "        enc2 = self.middle(enc1)\n",
    "        bottleneck = self.bottleneck(enc2)\n",
    "\n",
    "        up1 = self.upconv1(bottleneck)\n",
    "        merge1 = torch.cat([enc2, up1], dim=1)\n",
    "        dec1 = self.decoder1(merge1)\n",
    "\n",
    "        up2 = self.upconv2(dec1)\n",
    "        merge2 = torch.cat([enc1, up2], dim=1)\n",
    "        dec2 = self.decoder2(merge2)\n",
    "\n",
    "        output = self.output_conv(dec2)\n",
    "        return output\n"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.8.16"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
